from __future__ import absolute_import

import pytest

from blurhash import encode


def test_encode_file():
    with open('tests/pic2.png', 'rb') as image_file:
        result = encode(image_file, 4, 3)

    assert result == 'LlMF%n00%#MwS|WCWEM{R*bbWBbH'


def test_encode_with_filename():
    result = encode('tests/pic2.png', 4, 3)
    assert result == 'LlMF%n00%#MwS|WCWEM{R*bbWBbH'


def test_encode_black_and_white_picture():
    result = encode('tests/pic2_bw.png', 4, 3)
    assert result == 'LjIY5?00?bIUofWBWBM{WBofWBj['


def test_invalid_image():
    with pytest.raises(IOError):
        encode('README.md', 4, 3)


def test_file_does_not_exist():
    with pytest.raises(IOError):
        encode('pic404.png', 4, 3)


def test_invalid_x_components():
    with pytest.raises(ValueError):
        encode('tests/pic2.png', 10, 3)

    with pytest.raises(ValueError):
        encode('tests/pic2.png', 0, 3)


def test_invalid_y_components():
    with pytest.raises(ValueError):
        encode('tests/pic2.png', 4, 10)

    with pytest.raises(ValueError):
        encode('tests/pic2.png', 4, 0)



from __future__ import absolute_import

import cffi
import sys


ffibuilder = cffi.FFI()
ffibuilder.set_source('blurhash._encode', '''
    const char* blurHashForPixels(int xComponents, int yComponents, int width,
                                  int height, uint8_t *rgb,
                                  size_t bytesPerRow);

    const char* create_hash_from_pixels(int x_components, int y_components,
                                       int width, int height, uint8_t* rgb,
                                       size_t bytes_per_row) {
        return blurHashForPixels(x_components, y_components, width, height,
                                 rgb, bytes_per_row);
    }
''', sources=['src/encode.c'], extra_compile_args=['-std=gnu99'] if sys.platform != 'win32' else [])
ffibuilder.cdef('''
    const char* create_hash_from_pixels(int x_components, int y_components,
                                       int width, int height, uint8_t* rgb,
                                       size_t bytes_per_row);
''')

if __name__ == '__main__':
    ffibuilder.compile()



from __future__ import absolute_import
from itertools import chain

from PIL import Image
from six.moves import zip

from ._encode import ffi as _ffi, lib as _lib
from ._version import version as __version__


__all__ = 'encode', '__version__',


def encode(image_file, x_components, y_components):
    image = Image.open(image_file).convert('RGB')
    red_band = image.getdata(band=0)
    green_band = image.getdata(band=1)
    blue_band = image.getdata(band=2)
    rgb_data = list(chain.from_iterable(zip(red_band, green_band, blue_band)))
    width, height = image.size
    image.close()

    rgb = _ffi.new('uint8_t[]', rgb_data)
    bytes_per_row = _ffi.cast('size_t', width * 3)
    width = _ffi.cast('int', width)
    height = _ffi.cast('int', height)
    x_components = _ffi.cast('int', x_components)
    y_components = _ffi.cast('int', y_components)

    result = _lib.create_hash_from_pixels(x_components, y_components, width,
                                          height, rgb, bytes_per_row)

    if result == _ffi.NULL:
        raise ValueError('Invalid x_components or y_components')

    return _ffi.string(result).decode()



#!/usr/bin/env python
from __future__ import absolute_import

from setuptools import setup


with open('README.md', 'r') as readme_file:
    long_description = readme_file.read()


tests_require = [
    'pytest',
]

setup(
    name='blurhash-python',
    description='BlurHash encoder implementation for Python',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://blurha.sh',
    use_scm_version=dict(
        write_to='src/blurhash/_version.py',
    ),
    author='Atte Lautanala',
    author_email='atte.lautanala@wolt.com',
    packages=['blurhash'],
    package_dir={'': 'src'},
    install_requires=[
        'cffi',
        'Pillow',
        'six',
    ],
    setup_requires=[
        'cffi',
        'setuptools-scm',
    ],
    cffi_modules=['src/build_blurhash.py:ffibuilder'],
    tests_require=tests_require,
    extras_require={
        'testing': tests_require,
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
